#ifndef AES_HPP
#define AES_HPP

#include <vector>
#include <cstdio>
#include <string>
#include <iostream>

// Byte and Byte Array types
typedef unsigned char byte;
typedef std::vector<byte> byteArray;

// Word and Word Array types
typedef unsigned int word;
typedef std::vector<word> wordArray;

// Exceptions
class incorrectKeySizeException { };
class incorrectTextSizeException { };
class badStateArrayException { };

class AES
{
	public:
		AES (const byteArray& key);

		byteArray encrypt (const byteArray& plaintext) const;
		byteArray decrypt (const byteArray& ciphertext) const;

	private:
		/* Block size in words -- Always constant in AES. */
		static const unsigned int Nb = 4;

		/* Key size in words -- can be 4, 6, or 8. */
		const unsigned int Nk;

		/* Number of rounds -- depends on key size. */
		const unsigned int Nr;

		wordArray keySchedule;

		void KeyExpansion (const byteArray& key, wordArray& w) const;
		word SubWord (const word& input) const;
		word RotWord (const word& input) const;
		wordArray GetRoundKey (const int round) const;

		void SubBytes (byteArray& state) const;
		void ShiftRows (byteArray& state) const;
		void MixColumns (byteArray& state) const;
		void AddRoundKey (byteArray& state, const wordArray& w) const;

		void InvSubBytes (byteArray& state) const;
		void InvShiftRows (byteArray& state) const;
		void InvMixColumns (byteArray& state) const;

};

#endif /* AES_HPP */

/* vim: set ts=4 sts=4 sw=4 noet tw=112: */
